// -*- mode: C++; c-file-style: "Stroustrup" -*-
//
// This file is part of krot,
// a program for the simulation, assignment and fit of HRLIF spectra.
//
// Copyright (C) 1998 Jochen Kpper
//
// $Id: constants.h,v 1.1 1998/09/15 10:42:48 jochen Exp $



#ifndef KROT_CONSTANTS_H
#define KROT_CONSTANTS_H



#include "krot.h"

#include <cmath>



/**
 * Define mathematical constants used throughout the program.
 *
 * If available, we use defines from <cmath>, otherwise we calculate everything
 * on our own.
 *
 * @author Jochen Kpper
 * @version $Id: constants.h,v 1.1 1998/09/15 10:42:48 jochen Exp $
 */
class Constants {

public:

    Constants() { KROT_LAUNCH1( "Launching Constants constructor - eps = %e", eps() ); };

    /**
     * Speed of light.
     */
    static const double c() { return _c; };

    /**
     * Conversion factor cm-1 -> MHz
     */
    static const double cm2MHz() { return _c * 100.0; };
    
    /**
     * Smallest floating point number for that 1.0 + eps > 1.0.
     */
    static const double eps();

    /**
     * ln( 2 )
     */      
    static const double ln2() { return _ln2; };

    /**
     * pi
     */      
    static const double pi() { return _pi; };

    /**
     * sqrt( ln( 2 ) )
     */      
    static const double sqrt_ln2() { return _sqrt_ln2; };
    
    /**
     * sqrt( ln( 2 ) / pi )
     */      
    static const double sqrt__ln2_div_pi() { return _sqrt__ln2_div_pi; };

    /**
     * sqrt( pi )
     */      
    static const double sqrt_pi() { return _sqrt_pi; };

private:
    
    static const double _c =           29979245.0L;
    static const double _ln2 =                0.693147180559945309417232121458L;
    static const double _pi =                 3.141592653589793238462643383280L;
    static const double _sqrt_ln2 =           0.832554611157697756353164644895L;
    static const double _sqrt__ln2_div_pi =   0.469718639349825666886170164205L;
    static const double _sqrt_pi =            1.772453850905516027298167483341L;
};



// some global utility functions
template<class TYPE>
inline TYPE sqr( const TYPE& x )
{
    return x * x;
}



template<class TYPE>
inline TYPE cube( const TYPE& x )
{
    return x * x * x;
}



#endif
