// -*- mode: C++; c-file-style: "Stroustrup" -*-
//
// This file is part of krot,
// a program for the simulation, assignment and fit of HRLIF spectra.
//
// Copyright (C) 1998 Jochen Kpper
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; see the file License. if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 , USA
//
// If you use this program for your scientific work, please cite it according to
// the file CITATION that comes with the package.
//
// $Id: spectrum.cc,v 1.5 1998/10/09 17:57:27 jochen Exp $



#include "dataset.h"
#include "krotData.h"
#include "rawData.h"
#include "spectrum.h"

#include <climits>
#include <fstream.h>
#include <iostream.h>
#include <string>
#include <vector>



Spectrum::Spectrum( KRotData *krotdata )
    : DataSet( krotdata )
{
    return;
}



Spectrum::Spectrum( const RawData& import, KRotData *krotData )
    : DataSet( krotData )
{
    delete [] data;
    data = import.linearize( dlta, strt, siz );
    mn = INT_MAX;
    mx = INT_MIN;
    for( unsigned int i=0; i<siz; i++ )
	if( data[i] > mx )
	    mx = data[i];
	else if( data[i] < mn )
	    mn = data[i];
    nam = import.name();    
    stp = strt + ( siz - 1 ) * dlta;
    cout << "start = " << strt << "\t"
	 << "stop  = " << stp << "\t"
	 << "min = " << mn  << "\t"
	 << "max = " << mx << endl;
//    for( unsigned int i=0; i<siz; i++ )
//	cout << data[i] << endl;
    return;
}



bool Spectrum::loadXY( const QString& fileName )
{
    KROT_LAUNCH1( "Launching Spectrum::loadXY with filename %s", fileName.data() );
    ifstream stream( fileName.data(), ios::in|ios::nocreate );
    if(! stream)
	return false;
    vector<double> x;
    vector<int> y;
    // read data
    while(! stream.eof()) {
	double  freq;
	int     inten;
	stream >> freq >> inten;
	x.push_back( freq );
	y.push_back( inten );
    }
    setName( fileName );
    siz = x.size() - 1;
    strt = static_cast<int>( x[0] );
    stp = strt + dlta * ( siz - 1 );
    mx = INT_MIN;
    mn = INT_MAX;
    delete [] data;
    data = new int64_t[siz];
    int     c  = 0;
    double  x1 = 0, x2 = 0;
    int     y1 = 0, y2 = 0;
    data[0] = y2 = y[c];
    x2 = x[c];
    for(unsigned int i=1; i<siz; i++) {
	double xc = strt + ( i * dlta );
	if( x2 < xc ) {
	    c++;
	    x1 = x2;    y1 = y2;
	    x2 = x[c];  y2 = y[c];
	}
	data[i] = static_cast<int>( y1 + ( xc - x1 ) * ( y2 -y1 ) / ( x2 - x1 ) );
	if( data[i] > mx )
	    mx = data[i];
	else if( data[i] < mn )
	    mn = data[i];
    }
#ifdef DEBUG_VERBOSE
    cerr << siz << " data points read." << endl
	 << "Startfrequenz: " << strt << ", delta: " << dlta << ", stop: " << stp << endl
	 << "niedrigste Intensitt: " << mn << "  hchste Intensitt: " << mx << endl << endl;
#endif
    return true;
}
